@@ -1,41 +1,49 @@
-## ----setup, echo=FALSE, include=FALSE---------------------------------------------------
+## ----setup, echo=FALSE, include=FALSE-------------------------------------
 library("knitr")
 # set global chunk options
 opts_chunk$set(prompt=TRUE)
-options(replace.assign=TRUE, width=90, prompt="R> ")
+options(prompt = "R> ", continue = "+  ", width = 76, useFancyQuotes = FALSE)
 opts_chunk$set(highlight=FALSE)
 opts_chunk$set(background='#FFFFFF')
 # opts_chunk$set(cache=TRUE, autodep=TRUE)
 opts_chunk$set(comment=NA)
+knit_hooks$set(document = function(x) {sub('\\usepackage[]{color}', '\\usepackage[usenames, dvipsnames]{xcolor}', x, fixed = TRUE)})
 
-## ----EpsInd,echo=FALSE, fig.width = 5, fig.height = 5, message=FALSE, warning=FALSE-----
+
+## ----EpsInd,echo=FALSE, fig.width = 5, fig.height = 5---------------------
+## Figure 1) Left
 library("GPareto", quietly = TRUE)
-ParetoRef <- matrix(c(0.2,0.22,0.35,0.5,0.65,0.8,0.7,0.6,0.55,0.4,0.25,0.2),6,2)
 
-## Epsilon indicateur
-plot(ParetoRef,xlim=c(0,1),ylim=c(0,1),pch=21, cex=2, col="black",
-     bg = "red", bty="l", xlab=expression(f[1]), ylab=expression(f[2])) #,xaxt="n",yaxt="n")
-plotParetoEmp(ParetoRef,col="red",lwd=2, lty = 2)
+# Pareto front creation
+ParetoRef <- matrix(c(0.2, 0.22, 0.35, 0.5, 0.65, 0.8, 0.7, 0.6, 0.55, 0.4, 0.25, 0.2), 6, 2)
+
+# Plot epsilon indicator
+plot(ParetoRef, xlim = c(0, 1), ylim = c(0, 1), pch = 21, cex = 2, col = "black", bg = "red", bty = "l",
+     xlab = expression(f[1]), ylab = expression(f[2]))
+plotParetoEmp(ParetoRef, col = "red", lwd = 2, lty = 2)
 
 points(x = 0.45, y = 0.32,  col = "black", bg = "blue", pch = 22, cex = 2)
-lines(c(0.45,0.45,0.65), c(0.55,0.32,0.32), lty = 5, lwd = 1, col = "blue")
+lines(c(0.45, 0.45, 0.65), c(0.55, 0.32, 0.32), lty = 5, lwd = 1, col = "blue")
 
 points(x = 0.1, y = 0.9,  col = "black", bg = "green", pch = 22, cex = 2)
-lines(c(0.1,0.1,0.2), c(1,0.9,0.9), lty = 5, lwd = 1, col = "green")
+lines(c(0.1, 0.1, 0.2), c(1, 0.9, 0.9), lty = 5, lwd = 1, col = "green")
 
-arrows(0.1,0.95,0.2,0.95, code = 3, length = 0.1, col = "green")
+arrows(0.1, 0.95, 0.2, 0.95, code = 3, length = 0.1, col = "green")
 
-arrows(0.55,0.32,0.55,0.4, code = 3, length = 0.1, col = "blue")
+arrows(0.55, 0.32, 0.55, 0.4, code = 3, length = 0.1, col = "blue")
 
-## ----HyperInd,echo=FALSE, fig.width = 5, fig.height = 5---------------------------------
-plot(ParetoRef,xlim=c(0,1),ylim=c(0,1),pch=21, cex=2, col="black",
-     bg = "red", bty="l", xlab=expression(f[1]), ylab=expression(f[2])) #,xaxt="n",yaxt="n")
-plotParetoEmp(ParetoRef,col="red",lwd=2, lty = 2)
+## ----HyperInd,echo=FALSE, fig.width = 5, fig.height = 5-------------------
+
+## Figure 1) right
+# Plot hypervolume indicator
+plot(ParetoRef, xlim = c(0, 1), ylim = c(0, 1), pch = 21, cex = 2, col = "black", bg = "red", bty = "l",
+     xlab = expression(f[1]), ylab = expression(f[2]))
+plotParetoEmp(ParetoRef, col = "red", lwd = 2, lty = 2)
 
 polygon(x = c(0.45, 0.45, 0.65, 0.65, 0.5, 0.5),
-        y = c(0.55, 0.32, 0.32, 0.4, 0.4, 0.55), col = rgb(0,0,1,0.5), border = NA)
+        y = c(0.55, 0.32, 0.32, 0.4, 0.4, 0.55), col = rgb(0, 0, 1, 0.5), border = NA)
 
-polygon(x = c(0.1, 0.1, 0.2, 0.2), y = c(1, 0.9, 0.9, 1), col = rgb(0,1,0,0.5), border =
+polygon(x = c(0.1, 0.1, 0.2, 0.2), y = c(1, 0.9, 0.9, 1), col = rgb(0, 1, 0, 0.5), border =
           NA)
 
 polygon(x = c(0.2, 0.2, 0.22, 0.22, 0.35, 0.35, 0.5, 0.5, 0.65, 0.65, 0.8, 0.8, 1, 1 ),
@@ -43,11 +51,11 @@ polygon(x = c(0.2, 0.2, 0.22, 0.22, 0.35, 0.35, 0.5, 0.5, 0.65, 0.65, 0.8, 0.8,
         density = 15, col = "red", border = NA)
 
 points(x = 0.45, y = 0.32,  col = "black", bg = "blue", pch = 22, cex = 2)
-lines(c(0.45,0.45,0.65), c(0.55,0.32,0.32), lty = 5, lwd = 1, col = "blue")
+lines(c(0.45, 0.45, 0.65), c(0.55, 0.32, 0.32), lty = 5, lwd = 1, col = "blue")
 
 
 points(x = 0.1, y = 0.9,  col = "black", bg = "green", pch = 22, cex = 2)
-lines(c(0.1,0.1,0.2), c(1,0.9,0.9), lty = 5, lwd = 1, col = "green")
+lines(c(0.1, 0.1, 0.2), c(1, 0.9, 0.9), lty = 5, lwd = 1, col = "green")
 
 points(x = 1, y = 1, pch = 13, cex = 2)
 
@@ -125,7 +133,7 @@ points(response1, response2, pch = 17)
 par(mfrow = c(1,1))
 rm(list = ls())
 
-## ----echo=FALSE, message=FALSE, warning=FALSE-------------------------------------------
+## ----echo=FALSE, message=FALSE, warning=FALSE-----------------------------
 library("DiceDesign", quietly = TRUE)
 set.seed(42)
 
@@ -139,14 +147,14 @@ I <- !is_dominated(t(F))
 Fstar <- F[I,]
 Xstar <- X[I]
 
-## ----results='hide', warning=FALSE------------------------------------------------------
+## ----results='hide', warning=FALSE----------------------------------------
 design.init <- matrix(seq(0, 1, length.out = 6), ncol = 1)
 response.init <- MOP2(design.init)
 mf1 <- km(~1, design = design.init, response=response.init[, 1])
 mf2 <- km(~1, design = design.init, response=response.init[, 2])
 model <- list(mf1, mf2)
 
-## ----echo=FALSE, warning=FALSE, results='hide'------------------------------------------
+## ----echo=FALSE, warning=FALSE, results='hide'----------------------------
 # Prediction
 ParetoFront.init <- t(nondominated_points(t(response.init)))
 p1 <- predict(object = mf1, newdata = X, checkNames = FALSE, type = "UK")
@@ -156,19 +164,19 @@ p2 <- predict(object = mf2, newdata = X, checkNames = FALSE, type = "UK")
 refPoint <- c(max(F[, 1]), max(F[, 2]))
 crit <- apply(X, 1, crit_EHI, model, critcontrol = list(refPoint = refPoint))
 
-## ----warning=FALSE----------------------------------------------------------------------
+## ----warning=FALSE--------------------------------------------------------
 res <- GParetoptim(model = model, fn = MOP2, crit = "EHI", 
-                   nsteps = 7, lower = 0, upper = 1, 
-                   critcontrol = list(refPoint = c(2, 2)))
+  nsteps = 7, lower = 0, upper = 1, 
+  critcontrol = list(refPoint = c(2, 2)))
 
-## ----echo=FALSE, warning=FALSE----------------------------------------------------------
+## ----echo=FALSE, warning=FALSE--------------------------------------------
 mf1.res <- res$lastmodel[[1]]
 mf2.res <- res$lastmodel[[2]]
 p1.res <- predict(mf1.res, newdata = X, checkNames = FALSE, type = "UK")
 p2.res <- predict(mf2.res, newdata = X, checkNames = FALSE, type = "UK")
 ParetoFront.res <- t(nondominated_points(t(cbind(mf1.res@y, mf2.res@y))))
 
-## ----fig.width=8, fig.height=6, echo=FALSE,warning=FALSE--------------------------------
+## ----fig.width=8, fig.height=6, echo=FALSE,warning=FALSE------------------
 # Plots
 par(mfrow=c(3,3))
 plot(X, F[,1], type = "l", main = "Objective 1", xlab = "x", ylab = expression(f[1]))
@@ -221,10 +229,10 @@ plotParetoEmp(ParetoFront.res, col="blue")
 par(mfrow=c(1,1))
 rm(list=ls())
 
-## ----P1,fig.show='hide',fig.width=10, fig.height=5--------------------------------------
+## ----P1,fig.show='hide',fig.width=10, fig.height=5------------------------
 plotParetoGrid(P1)
 
-## ----echo=TRUE,message=FALSE,warning=FALSE, results='hide'------------------------------
+## ----echo=TRUE,message=FALSE,warning=FALSE, results='hide'----------------
 set.seed(1)
 d <- 2; ninit <- 7; fun <- P1
 design <- lhsDesign(ninit, d, seed = 42)$design
@@ -233,28 +241,28 @@ mf1 <- km(~., design = design, response = response[, 1])
 mf2 <- km(~., design = design, response = response[, 2])
 model <- list(mf1, mf2)
 
-## ----echo=TRUE,message=FALSE,warning=FALSE,results='hide'-------------------------------
+## ----echo=TRUE,message=FALSE,warning=FALSE,results='hide'-----------------
 x.grid <- seq(0, 1, length.out = 21)
 test.grid <- expand.grid(x.grid, x.grid)
 SURcontrol <- list(integration.points = test.grid)
-omEGO1 <- crit_optimizer(crit = "SUR", model = model,  
-                         lower = c(0, 0), upper = c(1, 1),
-                         optimcontrol = list(method = "genoud", pop.size = 20,
-                                             int.seed = 2, unif.seed = 3),
-                         critcontrol = list(SURcontrol = SURcontrol))
+omEGO1 <- crit_optimizer(crit = "SUR", model = model,  lower = c(0, 0),
+  upper = c(1, 1), critcontrol = list(SURcontrol = SURcontrol),
+  optimcontrol = list(method = "genoud", pop.size = 20, 
+  int.seed = 2, unif.seed = 3)
+  )
 
-## ----echo=TRUE,message=FALSE,warning=FALSE, results='hide'------------------------------
+## ----echo=TRUE,message=FALSE,warning=FALSE, results='hide'----------------
 fun1 <- function(x) P1(x)[, 1]; fun2 <- function(x) P1(x)[, 2]
 fastmf2 <- fastfun(fn = fun2, design = design, response = response[, 2])
 model2 <- list(mf1, fastmf2)
 
-## ----echo=FALSE,message=FALSE,warning=FALSE, results='hide'-----------------------------
+## ----echo=FALSE,message=FALSE,warning=FALSE, results='hide'---------------
 omEGO2 <- crit_optimizer(crit="SUR", model=model2,  lower=c(0,0), upper=c(1,1),
                          optimcontrol=list(method="pso", maxit=50),
                          critcontrol=list(SURcontrol=SURcontrol))
 
-## ----echo=FALSE,message=FALSE,warning=FALSE, results='hide'-----------------------------
-library("KrigInv", quietly = TRUE)
+## ----echo=FALSE,message=FALSE,warning=FALSE, results='hide'---------------
+suppressPackageStartupMessages(library("KrigInv", quietly = T))
 n.grid <- 21
 x.grid <- seq(0, 1, length.out=n.grid)
 
@@ -286,7 +294,7 @@ critcontrol2 <- list(integration.points = integration.points,
 SUR_grid <- apply(test.grid, 1, crit_SUR, model=model, critcontrol=critcontrol)
 SUR_grid2 <- apply(test.grid, 1, crit_SUR, model=model2, critcontrol=critcontrol2)
 
-## ----echo=FALSE,message=FALSE,warning=FALSE, fig.height=6, fig.width=6------------------
+## ----echo=FALSE,message=FALSE,warning=FALSE, fig.height=6, fig.width=6----
 filled.contour(x.grid, x.grid, 
                matrix(SUR_grid, n.grid), main="SUR Criterion",
                xlab=expression(x[1]), ylab=expression(x[2]), color=terrain.colors,
@@ -295,7 +303,7 @@ filled.contour(x.grid, x.grid,
                  points(omEGO1$par, col="red", pch=4, cex=2)})
 
 
-## ----echo=FALSE,message=FALSE,warning=FALSE, fig.height=6, fig.width=6------------------
+## ----echo=FALSE,message=FALSE,warning=FALSE, fig.height=6, fig.width=6----
 filled.contour(x.grid, x.grid,
                matrix((SUR_grid2), n.grid), main="SUR Criterion (fastfun)",
                xlab=expression(x[1]), ylab=expression(x[2]), color=terrain.colors,
@@ -305,40 +313,37 @@ filled.contour(x.grid, x.grid,
                }
 )
 
-## ----UQ_opt1, message = FALSE, warning=FALSE, results='hide', fig.show='hide'-----------
+## ----UQ_opt1, message = FALSE, warning=FALSE, results='hide', fig.show='hide'----
 sol <- GParetoptim(model = model, fn = fun, crit = "SUR", nsteps = 7,
-                   lower = c(0, 0), upper = c(1, 1), 
-                   optimcontrol = list(method = "pso"),
-                   critcontrol = list(SURcontrol = list(distrib = "SUR",
-                                                        n.points = 50)))
+  lower = c(0, 0), upper = c(1, 1), optimcontrol = list(method = "pso"),
+  critcontrol = list(SURcontrol = list(distrib = "SUR", n.points = 50)))
 
-## ----UQ_opt2, message=FALSE, warning=FALSE, results='hide', fig.show='hide'-------------
-solFast <- GParetoptim(model = list(mf1), fn = fun1, cheapfn = fun2,
-                   crit = "SUR", nsteps = 7, lower = c(0, 0), upper = c(1, 1),
-                   optimcontrol = list(method = "pso"),
-                   critcontrol = list(SURcontrol = list(distrib = "SUR",
-                                                    n.points = 50)))
+## ----UQ_opt2, message=FALSE, warning=FALSE, results='hide', fig.show='hide'----
+solFast <- GParetoptim(model = list(mf1), fn = fun1, cheapfn = fun2, 
+  crit = "SUR", nsteps = 7, lower = c(0, 0), upper = c(1, 1),
+  optimcontrol = list(method = "pso"),
+  critcontrol = list(SURcontrol = list(distrib = "SUR", n.points = 50)))
 
-## ----UQ_1, warning=FALSE, fig.show='hide',fig.width=6, fig.height=5---------------------
+## ----UQ_1, warning=FALSE, fig.show='hide',fig.width=6, fig.height=5-------
 lim1 <- seq(-50, 240, length.out = 41); lim2 <- seq(-35, 0, length.out = 41)
 plotGPareto(sol, UQ_PF = TRUE, UQ_PS = TRUE, UQ_dens = TRUE,
             control = list(f1lim = lim1, f2lim = lim2))
 
-## ----UQ_2, warning=FALSE, fig.show='hide',fig.width=6, fig.height=5---------------------
+## ----UQ_2, warning=FALSE, fig.show='hide',fig.width=6, fig.height=5-------
 plotGPareto(solFast, UQ_PF = TRUE, UQ_PS = TRUE, UQ_dens = TRUE,
             control = list(f1lim = lim1, f2lim = lim2))
 
-## ----warning=FALSE, message=FALSE-------------------------------------------------------
+## ----warning=FALSE, message=FALSE-----------------------------------------
 newPoint <- getDesign(model = sol$lastmodel, target = c(42, -26),
-                      lower=c(0, 0), upper=c(1, 1))
+  lower = c(0, 0), upper = c(1, 1), optimcontrol = list(method = "pso"))
 newPoint
 
-## ----echo=TRUE,message=FALSE,warning=FALSE, results='hide'------------------------------
-res <- easyGParetoptim(fn = DTLZ2, budget = 50,
-                       lower = rep(0, 4), upper = rep(1, 4), control = list(maxit = 40))
+## ----echo=TRUE,message=FALSE,warning=FALSE, results='hide'----------------
+res <- easyGParetoptim(fn = DTLZ2, budget = 50, lower = rep(0, 4), 
+  upper = rep(1, 4), control = list(maxit = 20))
 
-## ----ex3DPS, fig.show='hide',fig.width=c(5,6), fig.height=c(5,6)------------------------
-plotGPareto(res, UQ_PS = TRUE,
-            control = list(lower = rep(0, 4), upper = rep(1, 4), option = "mean",
-                           resolution = 25, nintegpoints = 100))
+## ----ex3DPS, fig.show='hide',fig.width=c(5,6), fig.height=c(5,6)----------
+plotGPareto(res, UQ_PS = TRUE, control = list(lower = rep(0, 4),
+  upper = rep(1, 4), nintegpoints = 100, option = "mean",
+  resolution = 25))
 
